//-----------------------------------------------------------------------------
/*!
\file IniLine.cpp
\brief Source for Line
\author Helmut Jakoby
\copyright © 1995 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the IniFile module.

IniFile is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General
Public License as published by the Free Software Foundation; either version 3 of
the License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "IniLine.h"
#include <cstring>
//-----------------------------------------------------------------------------
#ifdef _DEBUG_MEMORY_SPY
  #include <MilDebugNew.h>
#endif
//-------------------------------------------------------------------
namespace ini
{
  //-------------------------------------------------------------------
  #ifdef _MSC_VER
    //This function or variable may be unsafe
    #pragma warning( disable : 4996 )
  #endif
  //---------------------------------------------------------------------------
  Line::Line( const std::string & sCharLine )
  : m_sText(sCharLine),
    m_eLineType(IniUNDEFINED)
  {
  }
  //---------------------------------------------------------------------------
  Line::~Line ()
  {
  }
  //---------------------------------------------------------------------------
  Line::Line( const std::string & sCharLine, LINE_TYPE eLineType )
  : m_sText(sCharLine),
    m_eLineType(eLineType)
  {
  }
  //---------------------------------------------------------------------------
   Line::Line()
  : m_sText(NULL_PTR),
    m_eLineType(IniUNDEFINED)
  {
  }
  //---------------------------------------------------------------------------
  std::string Line::getText() const
  {
    return m_sText;
  }
  //---------------------------------------------------------------------------
  void Line::setText( std::string sCharLine )
  {
    if ( m_eLineType == IniUNDEFINED )
    {
      m_LineFactory.trim( sCharLine );
      // Typ ermitteln
      m_eLineType = m_LineFactory.getLineType( sCharLine );
    }
    m_sText = sCharLine;
  }
  //---------------------------------------------------------------------------
  LINE_TYPE Line::getType() const
  {
    return m_eLineType;
  }
  //---------------------------------------------------------------------------
  std::size_t Line::getLineSizeWithEOL() const
  {
    return m_sText.length() + ini::g_sEOL.length();
  }
  //---------------------------------------------------------------------------
  char * Line::getTextWithEOL() const
  {
    std::string   sRetValString = m_sText;
                  sRetValString += ini::g_sEOL;
    char        *  t_szRetVal = new char[std::strlen( sRetValString.c_str() ) + ini::g_sEOL.length()];

    std::memcpy( t_szRetVal, sRetValString.c_str(), sRetValString.length() );
    return t_szRetVal;
  }
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    //This function or variable may be unsafe
    #pragma warning( default : 4996 )
  #endif
  //---------------------------------------------------------------------------
} // namespace ini
