//-----------------------------------------------------------------------------
/*!
\file IniSection.cpp
\brief Source for Section
\author Helmut Jakoby
\copyright © 1995 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the IniFile module.

IniFile is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General
Public License as published by the Free Software Foundation; either version 3 of
the License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "IniSection.h"
#include "EuBlob.h"
#include "IniKey.h"
//-----------------------------------------------------------------------------
namespace ini
{
  //---------------------------------------------------------------------------
  #define SUPER Line
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // This function or variable may be unsafe. Consider using strcpy_s instead. To disable deprecation, use _CRT_SECURE_NO_WARNINGS. See online help for details.
    #pragma warning( disable : 4996 )
    // Der Enumerationstyp "xxx" umfasst keine Bereichseinschränkung. Geben Sie "enum class" Vorrang vor "enum" (Enum.3)
    #pragma warning( disable : 26812 )
  #endif
  //---------------------------------------------------------------------------
  Section::Section( const std::string & sCharLine )
  : SUPER(sCharLine, IniSECTION),
    m_SektionList(ptl::RESPONSIBLE|ptl::UNIQUE)
  {
  }
  //---------------------------------------------------------------------------
  Section::~Section ()
  {
  }
  //---------------------------------------------------------------------------
  Section::Section( const std::string & sCharLine, LINE_TYPE eLineType )
  : SUPER(sCharLine, eLineType),
    m_SektionList(ptl::RESPONSIBLE|ptl::UNIQUE)
  {
  }
  //---------------------------------------------------------------------------
  Section::Section()
  : m_SektionList(ptl::RESPONSIBLE|ptl::UNIQUE)
  {
  }
  //---------------------------------------------------------------------------
  std::string Section::getName() const
  {
    std::string   t_sRetVal = m_sText;
    std::size_t   t_Position = t_sRetVal.find( "[" );

    if ( t_Position != std::string::npos )
    {
      t_sRetVal.erase( t_Position, 1 );
    }
    t_Position = t_sRetVal.find( "]" );
    if ( t_Position != std::string::npos )
    {
      t_sRetVal.erase( t_Position, 1 );
    }
    return t_sRetVal;
  }
  //---------------------------------------------------------------------------
  void Section::setName( const std::string & sName )
  {
    m_sText = "[";

    m_sText += sName;
    m_sText += "]";
  }
  //---------------------------------------------------------------------------
  int Section::getSectionLine(  Line *& prRetVal, 
                                std::size_t nPosition,
                                ptl::SeekMode eMode )
  {
    return m_SektionList.get( prRetVal, nPosition, eMode );
  }
  //---------------------------------------------------------------------------
  int Section::removeSectionLine (ptl::ElementPosition eFirstLastCurrent)
  {
    return m_SektionList.remove( eFirstLastCurrent );
  }
  //---------------------------------------------------------------------------
  void Section::removeAllSectionLines ()
  {
    m_SektionList.removeAll();
  }
  //---------------------------------------------------------------------------
  int Section::insertSectionLine(  Line * pIniLine, 
                                   ptl::InsertPosition eInsertPosition, 
                                   ptl::VH eBeforeBehind )
  {
    return m_SektionList.insert(pIniLine, eInsertPosition, eBeforeBehind);
  }
  //---------------------------------------------------------------------------
  std::size_t Section::getSectionSize()
  {
    int            t_iErr = 0;
    Line        * t_pIniLine = NULL_PTR;
    std::size_t   t_nBlobSize = this->getLineSizeWithEOL();
  
    // ersten Eintrag holen
    t_iErr = this->getSectionLine(t_pIniLine, 0, ptl::START);
    if ( ! t_iErr )
    {
      do
      {
        t_nBlobSize += t_pIniLine->getLineSizeWithEOL();
        // nächsten Eintrag holen
        t_iErr = this->getSectionLine(t_pIniLine, 1, ptl::CURRENT);
      } while ( ! t_iErr );
    }
    return t_nBlobSize;
  }
  //---------------------------------------------------------------------------
  int Section::getKeyLine( Line *& prRetVal, const std::string & sValueVariable )
  {
    int     t_iErr = 0;
    Line  * t_pIniLine = NULL_PTR;
  
    prRetVal = NULL_PTR;
    t_iErr = this->getSectionLine( t_pIniLine, 0, ptl::START );
    if ( ! t_iErr )
    {
      do
      {
        // Nachsehen ob IniVARIABLE
        if (t_pIniLine->getType() == IniVARIABLE)
        {
          Key * t_pIniKey = dynamic_cast<Key*>( t_pIniLine );
          if ( t_pIniKey )
          {
            if ( sValueVariable == t_pIniKey->getKeyName() )
            {
              prRetVal = t_pIniLine;
            }
           }
          else
          {
            t_iErr = CAST;
          }
        }
        if ( ! t_iErr && ! prRetVal )
        {
          t_iErr = this->getSectionLine(t_pIniLine, 1, ptl::CURRENT);
        }
      } while ( ! t_iErr && ! prRetVal );
    }
    if ( t_iErr == ptl::RANGE )
    {
      t_iErr = KEY_NOT_EXISTING;
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  int Section::getKeyValue( std::string & rsValue, const std::string sValueVariable )
  {
    int     t_iErr = 0;
    Line  * t_pIniLine = NULL_PTR;
  
    rsValue = "";
    t_iErr = this->getKeyLine( t_pIniLine, sValueVariable );
    if ( ! t_iErr )
    {
      Key * t_pIniKey = dynamic_cast<Key*>( t_pIniLine );
      if ( t_pIniKey )
      {
        rsValue = t_pIniKey->getKeyValue();
      }
      else
      {
        t_iErr = CAST;
      }
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  int Section::insertSectionInBlob( eut::Blob & rBlob )
  {
    int      t_iErr = 0;
    Line  *  t_pIniLine = NULL_PTR;
    char  *  t_pTempBuffer = NULL_PTR;
  
    // zuerst eigenen Eintrag
    t_pTempBuffer = this->getTextWithEOL();
    t_iErr = rBlob.insert(  t_pTempBuffer,
                            this->getLineSizeWithEOL(),
                            rBlob.getSize() + 1
                          );
    delete[] t_pTempBuffer;
    if ( ! t_iErr )
    {
      // Einträge in Blob erstellen
      t_iErr = this->getSectionLine( t_pIniLine, 0, ptl::START );
      if ( ! t_iErr )
      {
        do
        {
          t_pTempBuffer = t_pIniLine->getTextWithEOL();
          t_iErr = rBlob.insert(  t_pTempBuffer,
                                  t_pIniLine->getLineSizeWithEOL(),
                                  rBlob.getSize() + 1
                                );
          delete[] t_pTempBuffer;
          if ( ! t_iErr )
          {
            t_iErr = this->getSectionLine( t_pIniLine, 1, ptl::CURRENT );
          }
        } while ( ! t_iErr );
      }
      if ( t_iErr == ptl::RANGE )
      {
        t_iErr = 0;
      }
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // This function or variable may be unsafe. Consider using strcpy_s instead. To disable deprecation, use _CRT_SECURE_NO_WARNINGS. See online help for details.
    #pragma warning( default : 4996 )
    // Der Enumerationstyp "xxx" umfasst keine Bereichseinschränkung. Geben Sie "enum class" Vorrang vor "enum" (Enum.3)
    #pragma warning( default : 26812 )
  #endif
  //---------------------------------------------------------------------------
  #undef SUPER
  //---------------------------------------------------------------------------
} // namespace ini
