#ifndef INC_EuTOOLERROR_CPP
#define INC_EuTOOLERROR_CPP
//-----------------------------------------------------------------------------
/*!
\file EuToolErrors.h
\brief
\if english
  For each library, here 'EuTools' there is an error file.
\endif

\if german
  Für jede Bibliothek, hier 'EuTools' gibt es eine Fehler-Datei.
\endif
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTools module.

EuTools is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
namespace eut
{
  //---------------------------------------------------------------------------
  /*!
  \enum ERRORS
  \brief
  \if english
    Error range = -14201 to -14300

    Warning range = 14301 to 14400
  
  \endif
  \if german
    Fehlerbereich = -14201 bis -14300

    Warnungbereich = 14301 bis 14400
  
  \endif
  */
  //---------------------------------------------------------------------------
  enum ERRORS
  {
    /*! -14201:
    \if english
      Opening a file has failed.
    \endif
  
    \if german
      Das Öffnen einer Datei ist fehlgeschlagen.
    \endif
    */
    ERR_OPEN_FILE = -14201,
    //-------------------------------------------------------------------------
    /*! -14202:
    \if english
      The file does not have the expected content.
    \endif
  
    \if german
      Die Datei hat nicht den erwarteten Inhalt.
    \endif
    */
    ERR_FILE_SEEK = -14202,
    //-------------------------------------------------------------------------
    /*! -14203:
    \if english
      The instantiation of an object failed.
    \endif
  
    \if german
      Das Instanziieren eine Objektes ist fehlgeschlagen.
    \endif
    */
    ERR_NEW = -14203,
    //-------------------------------------------------------------------------
    /*! -14204:
    \if english
      A range was exceeded.
    \endif
  
    \if german
      Ein Bereich wurde überschritten.
    \endif
    */
    ERR_RANGE = -14204,
    //-------------------------------------------------------------------------
    /*! -14205:
    \if english
      A file to be created already exists.
    \endif
  
    \if german
      Eine anzulegende Datei existiert bereits.
    \endif
    */
    ERR_FILE_ALREADY_EXIST = -14205,
    //-------------------------------------------------------------------------
    /*! -14206:
    \if english
      A directory to be created already exists.
    \endif
  
    \if german
      Ein anzulegendes Verzeichnis existiert bereits.
    \endif
    */
    ERR_DIR_ALREADY_EXIST = -14206,
    //-------------------------------------------------------------------------
    /*! -14207:
    \if english
      Calling the method is not allowed.
    \endif
  
    \if german
      Der Aufruf der Methode ist nicht erlaubt.
    \endif
    */
    ERR_METHOD_VORBIDDEN = -14207,
    //-------------------------------------------------------------------------
    /*! -14208:
    \if english
      The file extension is invalid.
    \endif
  
    \if german
      Die Datei-Erweiterung ist ungültig.
    \endif
    */
    ERR_FILE_EXTENSION = -14208,
    //-------------------------------------------------------------------------
    /*! -14209:
    \if english
      The expected file does not exist.
    \endif

    \if german
      Die erwartete Datei existiert nicht.
    \endif
    */
    ERR_FILE_DOES_NOT_EXIST = -14209,
    //-------------------------------------------------------------------------
    /*! -14210:
    \if english
      Object has the status NULL.
    \endif
  
    \if german
      Objekt hat den Status NULL.
    \endif
    */
    ERR_IS_NULL = -14210,
    //-------------------------------------------------------------------------
    /*! -14211:
    \if english
      The file path does not exist.
    \endif
  
    \if german
      Der Dateipfad existiert nicht.
    \endif
    */
    ERR_PATH_DOES_NOT_EXIST = -14211,
    //-------------------------------------------------------------------------
    /*! -14212:
    \if english
      The specified buffer size has been exceeded.
    \endif
  
    \if german
      Die vorgegebene Puffergröße wurde überschritten.
    \endif
    */
    ERR_BUFFER_OVERFLOW = -14212,
    //-------------------------------------------------------------------------
    /*! -14213:
    \if english
      A passed parameter must not be NULL or 0 or empty.
    \endif
  
    \if german
      Ein übergebener Parameter darf nicht NULL bzw. 0 oder leer sein.
    \endif
    */
    ERR_PARAMETER_NULL = -14213,
    //-------------------------------------------------------------------------
    /*! -14214:
    \if english
      A non-purvirtual method is by definition marked as such.
    \endif
  
    \if german
      Eine nichtpurvirtuale Methode ist per Definition als solche gekennzeichnet.
    \endif
    */
    ERR_PUR_VIRTUAL_FUNC = -14214,
    //-------------------------------------------------------------------------
    /*! -14215:
    \if english
      The source file does not exist.
    \endif

    \if german
      Die Quell-Datei existiert nicht.
    \endif
    */
    ERR_SOURCE_FILE_DOES_NOT_EXIST = -14215,
    //-------------------------------------------------------------------------
    /*! -14216:
    \if english
      The target file does not exist.
    \endif

    \if german
      Die Ziel-Datei existiert nicht.
    \endif
    */
    ERR_TARGET_FILE_DOES_NOT_EXIST = -14216,
    //-------------------------------------------------------------------------
    /*! -14217:
    \if english
      Unknown compiler.
    \endif

    \if german
      Unbekannter Compiler.
    \endif
    */
    ERR_UNKNOWN_COMPILER = -14217,
    //-------------------------------------------------------------------------
    /*! -14218:
    \if english
      Type(s) are not compatible or not correct in the context.
    \endif
  
    \if german
      Typ(en) ist/sind nicht kompatibel oder stimmen im Kontext nicht.
    \endif
    */
    ERR_TYP_MISMATCH = -14218,
    //-------------------------------------------------------------------------
    /*! -14219:
    \if english
      Passed parameters do not fit logically.
    \endif
  
    \if german
      Übergebene Parameter passen logisch nicht.
    \endif
    */
    ERR_PARAMETER_MISMATCHING = -14219,
    //-------------------------------------------------------------------------
    /*! -14220:
    \if english
      Too few parameters were passed.
    \endif
  
    \if german
      Es wurden zu wenig Parameter übergeben.
    \endif
    */
    ERR_PARAMETER_NUMBER = -14220,
    //-------------------------------------------------------------------------
    /*! -14221:
    \if english
      The passed object is already in the ObjectSpy.
    \endif
  
    \if german
      Das übergebene Objekt ist schon im ObjectSpy.
    \endif
    */
    ERR_OBJ_IN_OBJSPY = -14221,
    //-------------------------------------------------------------------------
    /*! -14222:
    \if english
      The passed object is not in the ObjectSpy.
    \endif
  
    \if german
      Das übergebene Objekt ist nicht im ObjectSpy.
    \endif
    */
    ERR_OBJ_NOT_IN_OBJSPY = -14222,
    //-------------------------------------------------------------------------
    /*! -14223:
    \if english
      An insert has failed.
    \endif
  
    \if german
      Ein Einfügen ist fehlgeschlagen.
    \endif
    */
    ERR_INSERT = -14223,
    //-------------------------------------------------------------------------
    /*! -14224:
    \if english
      A passed value is of the wrong type.
    \endif

    \if german
      Ein übergebener Wert ist vom falschen Typ.
    \endif
    */
    ERR_WRONG_TYPE = -14224,
    //-------------------------------------------------------------------------
    /*! -14225:
    \if english
      A specified value was exceeded.
    \endif
  
    \if german
      Ein vorgegebener Wert wurde überschritten.
    \endif
    */
    ERR_OVERFLOW = -14225,
    //-------------------------------------------------------------------------
    /*! -14226:
    \if english
      The value has fallen below a specified value.
    \endif
  
    \if german
      Ein vorgegebener Wert wurde unterschritten.
    \endif
    */
    ERR_UNDERFLOW = -14226,
    //-------------------------------------------------------------------------
    /*! -14227:
    \if english
      It could not be determined whether it is a 32 or 64 bit program.
    \endif
  
    \if german
      Es konnte nicht ermittelt werden, ob es sich um ein 32 oder 64 Bit-Programm handelt.
    \endif
    */
    ERR_KNOWN_ENV_BITS = -14227,
    //-------------------------------------------------------------------------
    /*! -14228:
    \if english
      A file to be created already exists as a directory.
    \endif
  
    \if german
      Eine anzulegende Datei besteht schon als Verzeichnis.
    \endif
    */
    ERR_FILENAME_ALREADY_EXISTS_AS_DIRECTORYNAME = -14228,
    //-------------------------------------------------------------------------
    /*! -14229:
    \if english
      A directory to be created already exists as a file.
    \endif
  
    \if german
      Ein anzulegendes Verzeichnis besteht schon als Datei.
    \endif
    */
    ERR_DIRECTORYNAME_ALREADY_EXISTS_AS_FILENAME = -14229,
    //-------------------------------------------------------------------------
    /*! -14230:
    \if english
      A directory could not be created.
    \endif
  
    \if german
      Ein Verzeichnis konnte nicht angelegt werden.
    \endif
    */
    ERR_DIRECTORY_COULD_NOT_BE_CREATED = -14230,
    //-------------------------------------------------------------------------
    /*! -14231:
    \if english
      A source value is equal to the target value.
    \endif
  
    \if german
      Ein Quell-Wert ist gleich dem Ziel-Wert.
    \endif
    */
    ERR_SOURCE_EQUAL_TARGET = -14231,
    //-------------------------------------------------------------------------
    /*! -14232:
    \if english
      Access was denied.
    \endif
  
    \if german
      Der Zugriff wurde verweigert.
    \endif
    */
    ERR_ACCESS_DENIED = -14232,
    //-------------------------------------------------------------------------
    /*! -14233:
    \if english
      Wildcard characters are not allowed in the current context.
    \endif
  
    \if german
      Platzhalterzeichen sind im aktuellen Kontext nicht erlaubt.
    \endif
    */
    ERR_WILDCARD_NOT_ALLOWED = -14233,
    //-------------------------------------------------------------------------
    /*! -14234:
    \if english
      Wildcard characters in the first filöename and a target filename are
      not allowed in the current context.
    \endif
  
    \if german
      Platzhalterzeichen im ersten Dateinamen und ein Zieldateiname sind im
      aktuellen Kontext nicht erlaubt.
    \endif
    */
    ERR_WILDCARD_IN_FIRST_FILE_AND_SECOND_FILE_NOT_ALLOWED = -14234,
    //-------------------------------------------------------------------------
    /*! -14235:
    \if english
      A file to be locked is already locked.
    \endif
  
    \if german
      Eine zu sperrende Datei ist schon gesperrt.
    \endif
    */
    ERR_FILE_LOCKED = -14235,
    //-------------------------------------------------------------------------
    /*! -14236:
    \if english
      Too many open files.
    \endif
  
    \if german
      Zu viele geöffnete Dateien.
    \endif
    */
    ERR_TO_MANY_OPEN_FILES = -14236,
    //-------------------------------------------------------------------------
    /*! -14237:
    \if english
      The first file does not exist.
    \endif

    \if german
      Die erste Datei existiert nicht.
    \endif
    */
    ERR_FIRST_FILE_NOT_EXIST = -14237,
    //-------------------------------------------------------------------------
    /*! -14238:
    \if english
      The second file does not exist.
    \endif

    \if german
      Die zweite Datei existiert nicht.
    \endif
    */
    ERR_SECOND_FILE_NOT_EXIST = -14238,
    //-------------------------------------------------------------------------
    /*! -14300:
    \if english
      Undefined error.
    \endif
  
    \if german
      Nicht definierter Fehler.
    \endif
    */
    ERR_UNKNOWN = -14300,
    //-------------------------------------------------------------------------
    
    //-------------------------------------------------------------------------
    // Warnings
    //-------------------------------------------------------------------------
    /*! -14301:
    \if english
      A passed value is of the wrong type.
    \endif

    \if german
      Ein übergebener Wert ist vom falschen Typ.
    \endif
    */
    WRN_WRONG_TYPE = -14301,
    //-------------------------------------------------------------------------
    /*! 13400:
    \if english
      Undefined warning.
    \endif

    \if german
      Nicht definierte Warnung.
    \endif
    */
    WRN_UNKNOWN = 14400
    //-------------------------------------------------------------------------
  };
} // namespace eut
#endif
