//-----------------------------------------------------------------------------
/*!
\file PtlAVDVList.cpp
\brief Source for AVDVList
\author Helmut Jakoby
\copyright © 1995 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the PtlContainer module.

PtlContainer is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "PtlAVDVList.h"
#include "EuException.h"
#include <new>
//-----------------------------------------------------------------------------
#ifdef _DEBUG_MEMORY_SPY
  #include <MilDebugNew.h>
#endif
//-----------------------------------------------------------------------------
namespace ptl
{
  //---------------------------------------------------------------------------
  AVDVList::AVDVList( ListMode eMode )
  : m_eListMode(eMode),
    m_iErrorCode(0),
    m_pFirstDVElement(NULL_PTR),
    m_pLastDVElement(NULL_PTR),
    m_nSize(0)
  {
  
  }
  //---------------------------------------------------------------------------
  AVDVList::~AVDVList()
  {
    removeAll();
  }
  //---------------------------------------------------------------------------
  AVDVList::AVDVList( const AVDVList& rList )
  : m_iErrorCode(0),
    m_pFirstDVElement(NULL_PTR),
    m_pLastDVElement(NULL_PTR),
    m_nSize(0)
  {
    operator=( rList );
  }
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // Der Enumerationstyp "xxx" umfasst keine Bereichseinschränkung. Geben Sie "enum class" Vorrang vor "enum" (Enum.3)
    #pragma warning( disable : 26812 )
  #endif
  //---------------------------------------------------------------------------
  void * AVDVList::get( ElementPosition eElementPos )
  {
    void  *  t_pRetVal = NULL_PTR;
  
    if (m_pFirstDVElement)  // wenn Liste nicht leer
    {
      // Wenn FIRST_POS...
      if (eElementPos == FIRST_POS)
      {
        // hole Datebobjekt vom ersten Element und gebe zurück
        t_pRetVal = m_pFirstDVElement->m_pData;
      }
      else
      {
        // hole Datebobjekt vom letzten Element und gebe zurück
        t_pRetVal = m_pLastDVElement->m_pData;
      }
    }
    // wenn Liste leer
    return t_pRetVal;
  }
  //---------------------------------------------------------------------------
  int AVDVList::insert( void* pDataObject,
                        InsertPosition eWhere,
                        VH eBeforeBehind )
  {
    int          t_iErr = 0;
    DVElement * t_pPosElement = NULL_PTR;
  
    // PositionsElement für Übergabe an insertSrc(...)
    if (eWhere == InsertHEAD)
    {
      t_pPosElement = m_pLastDVElement;
    }
    else
    {
      t_pPosElement = m_pFirstDVElement;
    }
    if (pDataObject)    // weiter nur wenn gültiges pDataObject übergeben wurde
    {
      // Wenn einfügen klappte; DVElement geliefert wird...
      if ( ( this->insertSrc( pDataObject, t_pPosElement, eBeforeBehind ) ) != NULL_PTR )
      {
        t_iErr = 0;
      }
      else
      {
        // Wenn Fehler, dann instanziierung nicht geklappt
        if (m_iErrorCode)
        {
          t_iErr = INSTANTIATE_DVELEMENT;  // instanziieren von DVElement ging nicht
        }
        else
        {
          t_iErr = OBJECT_ALREADY_AVAILABLE;  // Kein Einfügen; Datenobjekt schon drinn (nur bei UNIQUE)
        }
      }
    }
    else
    {
      t_iErr = PASSED_NO_OBJECT;    // kein pDataObject übergeben
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // Der Enumerationstyp "xxx" umfasst keine Bereichseinschränkung. Geben Sie "enum class" Vorrang vor "enum" (Enum.3)
    #pragma warning( default : 26812 )
  #endif
  //---------------------------------------------------------------------------
  size_t AVDVList::getSize()
  {
    return m_nSize;
  }
  //---------------------------------------------------------------------------
  size_t AVDVList::getNumberDataObjects( void* pDataObject )
  {
    DVElement * t_pIterElement = m_pFirstDVElement;
    size_t      t_nObjectCount = 0;
  
    if (t_pIterElement)
    {
      do
      {
        // Wenn Datenobjekt vorhanden, t_iObjectCount hochzählen.
        if ((t_pIterElement->m_pData) == pDataObject)
        {
          t_nObjectCount++;
        }
        // t_pIterElement auf nächstes DVElement
        t_pIterElement = t_pIterElement->m_pNextElement;
      } while (t_pIterElement);
    }
    return t_nObjectCount;
  }
  //---------------------------------------------------------------------------
  void AVDVList::removeAll()
  {
    while ( m_pFirstDVElement )
    {
      DVElement * t_pDelElement = m_pFirstDVElement;  // Zeiger auf erstes Element
      // Nächstes Element
      m_pFirstDVElement = m_pFirstDVElement->m_pNextElement;
      delete t_pDelElement;  // altes erstes Element löschen
    }  // Ende while
    // Letztes DVElement auf 0
    m_pLastDVElement = NULL_PTR;
    // Anzahl der VElemente auf 0 setzen
    m_nSize = 0;
  }
  //---------------------------------------------------------------------------
  int AVDVList::getErrorCode()
  {
    return m_iErrorCode;
  }
  //---------------------------------------------------------------------------
  ListMode AVDVList::getListMode()
  {
    return m_eListMode;
  }
  //---------------------------------------------------------------------------
  int AVDVList::_addElementFrom( const AVDVList& rList )
  {
    int t_iErr = 0;
  
    // t_pIterElement wird erstes DVElement von übergebener Liste
    DVElement *  t_pIterElement = rList.m_pFirstDVElement;
  
    if (this != &rList)
    {
      // neuen Elemente aus übergebener Liste zufügen wenn t_pIterElement
      if (t_pIterElement)
      {
        do
        {
          // Wenn Datenobjekt holen und in Liste anhängen geht...
          if (this->insert(t_pIterElement->m_pData, InsertLAST, BEHIND) != INSTANTIATE_DVELEMENT)
          {
            // t_pIterElement auf nächstes DVElement
            t_pIterElement = t_pIterElement->m_pNextElement;
          }
          else  // Abbruch wenn instanziieren nicht geht
          {
            t_pIterElement = NULL_PTR;
            t_iErr = INSTANTIATE_DVELEMENT;
          }
        } while (t_pIterElement);
      }
    }
    else
    {
      t_iErr = SAME_SOURCE_TARGET;
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  DVElement * AVDVList::insertSrc(  void * pDataObject,
                                    DVElement* pReferenceDVElement,
                                    VH eBeforeBehind)
  {
    DVElement  * t_NewDVElement = NULL_PTR;
  
    // ErrorCode auf 0
    m_iErrorCode = 0;
  
    // Wenn UNIQUE-Liste und Datenobjekt noch nicht in Liste oder DEFAULT-Liste
    if ((((m_eListMode & UNIQUE) == UNIQUE) && ( ! getNumberDataObjects( pDataObject ) ) )
          || ((m_eListMode & UNIQUE) == 0))
    {
      // Neues DVElement anlegen und prüfen auf Gelingen
      #ifdef _DEBUG_MEMORY_SPY
        t_NewDVElement = new DVElement( pDataObject, NULL_PTR, NULL_PTR );
      #else
        t_NewDVElement = new (std::nothrow) DVElement( pDataObject, NULL_PTR, NULL_PTR );
      #endif
      if ( t_NewDVElement )
      {
        // Anzahl hochzählen
        m_nSize++;
        // Wenn pReferenceDVElement übergeben, alten Vorgänger und Nachfolger aufheben
        if (pReferenceDVElement)
        {
          if (eBeforeBehind == BEFORE)    // Wenn an pReferenceDVElement vorhängen
          {
            // Beim neuen Element den Nachfolger auf Referenzelement setzen
            t_NewDVElement->m_pNextElement = pReferenceDVElement;
            // Beim neuen Element den Vorgänger auf den Vorgänger des
            // Referenzelements setzen
            t_NewDVElement->m_pPreviousElement = pReferenceDVElement->m_pPreviousElement;
            // Beim Vorgänger des Referenzelements den Nachfolger auf neues Element
            // setzen, wenn dieser vorhanden
            if (pReferenceDVElement->m_pPreviousElement)
            {
              pReferenceDVElement->m_pPreviousElement->m_pNextElement = t_NewDVElement;
            }
            // Beim Referenzelement den Vorgänger auf neues Element setzen
            pReferenceDVElement->m_pPreviousElement = t_NewDVElement;
            // Wenn pReferenceDVElement m_pFirstDVElement war, wird t_NewDVElement
            // zu m_pFirstDVElement
            if (pReferenceDVElement == m_pFirstDVElement)
            {
              m_pFirstDVElement = t_NewDVElement;
            }
          }
          else  // Wenn an pReferenceDVElement anhängen
          {
            // Beim neuen Element den Nachfolger auf den Nachfolger des
            // Referenzelement setzen
            t_NewDVElement->m_pNextElement = pReferenceDVElement->m_pNextElement;
            // Beim neuen Element den Vorgänger auf das Referenzelements setzen
            t_NewDVElement->m_pPreviousElement = pReferenceDVElement;
            // Beim Nachfolger des Referenzelements den Vorgänger auf neues Element
            // setzen, wenn dieser vorhanden
            if (pReferenceDVElement->m_pNextElement)
            {
              pReferenceDVElement->m_pNextElement->m_pPreviousElement = t_NewDVElement;
            }
            // Beim Referenzelement den Nachfolger auf neues Element setzen
            pReferenceDVElement->m_pNextElement = t_NewDVElement;
            // Wenn pReferenceDVElement m_pLastDVElement war, wird t_NewDVElement
            // zu m_pLastDVElement
            if (pReferenceDVElement == m_pLastDVElement)
            {
              m_pLastDVElement = t_NewDVElement;
            }
          }
        }
        else  // ansonsten (Liste leer), m_pLastDVElement und m_pFirstDVElement
              // setzen.
        {
          m_pFirstDVElement = t_NewDVElement;
          m_pLastDVElement = t_NewDVElement;
        }
      }
      else // instanziieren von DVElement geht nicht
      {
        m_iErrorCode = INSTANTIATE_DVELEMENT;
      }
    }
    return t_NewDVElement;
  }
  //---------------------------------------------------------------------------
  DVElement * AVDVList::removeSrc( DVElement * pDeadDVElement )
  {
    DVElement * t_pRetVal = NULL_PTR;
  
    // Wenn DVElement übergeben
    if (pDeadDVElement)
    {
      // Wenn Vorgänger von zu löschendem DVElement vorhanden
      if (pDeadDVElement->m_pPreviousElement)
      {
        // bei Vorgänger-Nachfolger auf zu löschendes DVElement-Nachfolger setzen
        pDeadDVElement->m_pPreviousElement->m_pNextElement = pDeadDVElement->m_pNextElement;
        // und Vorgänger wird erstmal Rückgabwert
        t_pRetVal = pDeadDVElement->m_pPreviousElement;
      }
      // Wenn kein Vorgänger von zu löschendem DVElement, wird dessen Nachfolger
      // erstes DVElement.
      else
      {
        m_pFirstDVElement = pDeadDVElement->m_pNextElement;
      }
      // Wenn Nachfolger von zu löschendem DVElement vorhanden
      if (pDeadDVElement->m_pNextElement)
      {
        // bei Nachfolger-Vorgänger auf zu löschendes DVElement-Vorgänger setzen
        pDeadDVElement->m_pNextElement->m_pPreviousElement = pDeadDVElement->m_pPreviousElement;
        // und Nachfolger wird zum Rückgabwert
        t_pRetVal = pDeadDVElement->m_pNextElement;
      }
      // Wenn kein Nachfolger von zu löschendem DVElement, wird dessen Vorgänger
      // letztes DVElement
      else
      {
        m_pLastDVElement = pDeadDVElement->m_pPreviousElement;
      }
      // löschen und m_nSize runter
      delete pDeadDVElement;
      m_nSize--;
    }
    return t_pRetVal;
  }
  //---------------------------------------------------------------------------
  AVDVList & AVDVList::operator= ( const AVDVList & rList )
  {
    if ( this != &rList )
    {
      // alle Element löschen um neue Elemente zu übertragen
      this->removeAll();
      int t_iErr = this->_addElementFrom(rList);
      if ( t_iErr )
      {
        throw eut::ErrorNException("Zufügen der Elemente gescheitert!", t_iErr, __FILE__, __LINE__);
      }
    }
    return *this;
  }
  //---------------------------------------------------------------------------
} // namespace ptl
